@@ -70,6 +70,7 @@ static errcode_t parse_int(struct field_set_info *info, char *field, char *arg);
 static errcode_t parse_string(struct field_set_info *info, char *field, char *arg);
 static errcode_t parse_uuid(struct field_set_info *info, char *field, char *arg);
 static errcode_t parse_hashalg(struct field_set_info *info, char *field, char *arg);
+static errcode_t parse_encoding(struct field_set_info *info, char *field, char *arg);
 static errcode_t parse_time(struct field_set_info *info, char *field, char *arg);
 static errcode_t parse_bmap(struct field_set_info *info, char *field, char *arg);
 static errcode_t parse_gd_csum(struct field_set_info *info, char *field, char *arg);
@@ -181,6 +182,7 @@ static struct field_set_info super_fields[] = {
 	{ "encrypt_pw_salt", &set_sb.s_encrypt_pw_salt, NULL, 16, parse_uuid },
 	{ "lpf_ino", &set_sb.s_lpf_ino, NULL, 4, parse_uint },
 	{ "checksum_seed", &set_sb.s_checksum_seed, NULL, 4, parse_uint },
+	{ "encoding", &set_sb.s_encoding, NULL, 2, parse_encoding },
 	{ 0, 0, 0, 0 }
 };
 
@@ -485,10 +487,7 @@ static errcode_t parse_uint(struct field_set_info *info, char *field,
 	n = num & mask;
 	switch (size) {
 	case 8:
-		/* Should never get here */
-		fprintf(stderr, "64-bit field %s has a second 64-bit field\n"
-			"defined; BUG?!?\n", info->name);
-		*u.ptr64 = 0;
+		*u.ptr64 = n;
 		break;
 	case 4:
 		*u.ptr32 = n;
@@ -508,7 +507,10 @@ static errcode_t parse_uint(struct field_set_info *info, char *field,
 		size = 2;
 	switch (size) {
 	case 8:
-		*u.ptr64 = n;
+		/* Should never get here */
+		fprintf(stderr, "64-bit field %s has a second 64-bit field\n"
+			"defined; BUG?!?\n", info->name);
+		*u.ptr64 = 0;
 		break;
 	case 4:
 		*u.ptr32 = n;
@@ -631,6 +633,19 @@ static errcode_t parse_hashalg(struct field_set_info *info,
 	return 0;
 }
 
+static errcode_t parse_encoding(struct field_set_info *info,
+				char *field EXT2FS_ATTR((unused)), char *arg)
+{
+	int	encoding;
+	unsigned char	*p = (unsigned char *) info->ptr;
+
+	encoding = e2p_str2encoding(arg);
+	if (encoding < 0)
+		return parse_uint(info, field, arg);
+	*p = encoding;
+	return 0;
+}
+
 static errcode_t parse_bmap(struct field_set_info *info,
 			    char *field EXT2FS_ATTR((unused)), char *arg)
 {
@@ -674,7 +689,6 @@ static errcode_t parse_inode_csum(struct field_set_info *info, char *field,
 	errcode_t	retval = 0;
 	__u32		crc;
 	int		is_large_inode = 0;
-	struct ext2_inode_large *tmp_inode;
 
 	if (strcmp(arg, "calc") == 0) {
 		size_t sz = EXT2_INODE_SIZE(current_fs->super);
