@@ -9,6 +9,7 @@
  * %End-Header%
  */
 
+#include "config.h"
 #include <errno.h>
 
 #include "e2fsck.h"
@@ -36,9 +37,12 @@ errcode_t e2fsck_allocate_context(e2fsck_t *ret)
 
 	time_env = getenv("E2FSCK_TIME");
 	if (time_env)
-		context->now = strtoul(time_env, NULL, 0);
-	else
+		context->now = (time_t) strtoull(time_env, NULL, 0);
+	else {
 		context->now = time(0);
+		if (context->now < 1262322000) /* January 1 2010 */
+			context->flags |= E2F_FLAG_TIME_INSANE;
+	}
 
 	*ret = context;
 	return 0;
@@ -52,7 +56,7 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 {
 	int	i;
 
-	ctx->flags = 0;
+	ctx->flags &= E2F_RESET_FLAGS;
 	ctx->lost_and_found = 0;
 	ctx->bad_lost_and_found = 0;
 	if (ctx->inode_used_map) {
@@ -85,9 +89,7 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 		ctx->fs->dblist = 0;
 	}
 	e2fsck_free_dir_info(ctx);
-#ifdef ENABLE_HTREE
 	e2fsck_free_dx_dir_info(ctx);
-#endif
 	if (ctx->refcount) {
 		ea_refcount_free(ctx->refcount);
 		ctx->refcount = 0;
@@ -96,6 +98,18 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 		ea_refcount_free(ctx->refcount_extra);
 		ctx->refcount_extra = 0;
 	}
+	if (ctx->ea_block_quota_blocks) {
+		ea_refcount_free(ctx->ea_block_quota_blocks);
+		ctx->ea_block_quota_blocks = 0;
+	}
+	if (ctx->ea_block_quota_inodes) {
+		ea_refcount_free(ctx->ea_block_quota_inodes);
+		ctx->ea_block_quota_inodes = 0;
+	}
+	if (ctx->ea_inode_refs) {
+		ea_refcount_free(ctx->ea_inode_refs);
+		ctx->ea_inode_refs = 0;
+	}
 	if (ctx->block_dup_map) {
 		ext2fs_free_block_bitmap(ctx->block_dup_map);
 		ctx->block_dup_map = 0;
@@ -104,6 +118,10 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 		ext2fs_free_block_bitmap(ctx->block_ea_map);
 		ctx->block_ea_map = 0;
 	}
+	if (ctx->block_metadata_map) {
+		ext2fs_free_block_bitmap(ctx->block_metadata_map);
+		ctx->block_metadata_map = 0;
+	}
 	if (ctx->inode_bb_map) {
 		ext2fs_free_inode_bitmap(ctx->inode_bb_map);
 		ctx->inode_bb_map = 0;
@@ -136,6 +154,18 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 		ext2fs_free_mem(&ctx->invalid_inode_table_flag);
 		ctx->invalid_inode_table_flag = 0;
 	}
+	if (ctx->encrypted_dirs) {
+		ext2fs_u32_list_free(ctx->encrypted_dirs);
+		ctx->encrypted_dirs = 0;
+	}
+	if (ctx->casefolded_dirs) {
+		ext2fs_u32_list_free(ctx->casefolded_dirs);
+		ctx->casefolded_dirs = 0;
+	}
+	if (ctx->inode_count) {
+		ext2fs_free_icount(ctx->inode_count);
+		ctx->inode_count = 0;
+	}
 
 	/* Clear statistic counters */
 	ctx->fs_directory_count = 0;
@@ -183,6 +213,19 @@ void e2fsck_free_context(e2fsck_t ctx)
 	if (ctx->device_name)
 		ext2fs_free_mem(&ctx->device_name);
 
+	if (ctx->log_fn)
+		free(ctx->log_fn);
+
+	if (ctx->logf)
+		fclose(ctx->logf);
+
+	if (ctx->problem_log_fn)
+		free(ctx->problem_log_fn);
+
+	if (ctx->problem_logf) {
+		fputs("</problem_log>\n", ctx->problem_logf);
+		fclose(ctx->problem_logf);
+	}
 	ext2fs_free_mem(&ctx);
 }
 
@@ -192,11 +235,9 @@ void e2fsck_free_context(e2fsck_t ctx)
  */
 typedef void (*pass_t)(e2fsck_t ctx);
 
-pass_t e2fsck_passes[] = {
-	e2fsck_pass1, e2fsck_pass2, e2fsck_pass3, e2fsck_pass4,
-	e2fsck_pass5, 0 };
-
-#define E2F_FLAG_RUN_RETURN	(E2F_FLAG_SIGNAL_MASK|E2F_FLAG_RESTART)
+static pass_t e2fsck_passes[] = {
+	e2fsck_pass1, e2fsck_pass1e, e2fsck_pass2, e2fsck_pass3,
+	e2fsck_pass4, e2fsck_pass5, 0 };
 
 int e2fsck_run(e2fsck_t ctx)
 {
@@ -214,6 +255,8 @@ int e2fsck_run(e2fsck_t ctx)
 	for (i=0; (e2fsck_pass = e2fsck_passes[i]); i++) {
 		if (ctx->flags & E2F_FLAG_RUN_RETURN)
 			break;
+		if (e2fsck_mmp_update(ctx->fs))
+			fatal_error(ctx, 0);
 		e2fsck_pass(ctx);
 		if (ctx->progress)
 			(void) (ctx->progress)(ctx, 0, 0, 0);
